extends Path

export var radius : float = 0.2
export var wall_height : float = 0.25
export var wall_thickness : float = 0.025
var st : SurfaceTool = SurfaceTool.new()

func _init():
	st.begin(Mesh.PRIMITIVE_TRIANGLES)
	var points : PoolVector3Array = curve.get_baked_points()
	var up_vecs : PoolVector3Array = curve.get_baked_up_vectors()
	if points.size() > 1:
		for i in points.size() - 2:
			var side_normal : Vector3 = (up_vecs[i]).cross(points[i] - points[i + 1]).normalized()
			var side_vec_wall : Vector3 = side_normal * (wall_thickness + radius)
			var side_vec : Vector3 = side_normal * radius
			var next_side_normal : Vector3 = (up_vecs[i + 1]).cross(points[i + 1] - points[i + 2]).normalized()
			var next_side_vec : Vector3 = next_side_normal * radius
			var next_side_vec_wall : Vector3 = next_side_normal * (wall_thickness + radius)
			
			# BOTTOM
			st.add_vertex(points[i + 1] + next_side_vec_wall - up_vecs[i + 1] * wall_thickness)
			st.add_vertex(points[i] - side_vec_wall - up_vecs[i] * wall_thickness)
			st.add_vertex(points[i] + side_vec_wall - up_vecs[i] * wall_thickness)
			st.add_vertex(points[i + 1] + next_side_vec_wall - up_vecs[i + 1] * wall_thickness)
			st.add_vertex(points[i + 1] - next_side_vec_wall - up_vecs[i + 1] * wall_thickness)
			st.add_vertex(points[i] - side_vec_wall - up_vecs[i] * wall_thickness)
			
			# RIGHT WALL OUTER
			st.add_vertex(points[i] + side_vec_wall + up_vecs[i] * wall_height)
			st.add_vertex(points[i + 1] + next_side_vec_wall + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i] + side_vec_wall - up_vecs[i] * wall_thickness)
			st.add_vertex(points[i + 1] + next_side_vec_wall + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i + 1] + next_side_vec_wall - up_vecs[i + 1] * wall_thickness)
			st.add_vertex(points[i] + side_vec_wall - up_vecs[i] * wall_thickness)
			
			# RIGHT WALL TOP
			st.add_vertex(points[i] + side_vec + up_vecs[i] * wall_height)
			st.add_vertex(points[i + 1] + next_side_vec_wall + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i] + side_vec_wall + up_vecs[i] * wall_height)
			st.add_vertex(points[i + 1] + next_side_vec + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i + 1] + next_side_vec_wall + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i] + side_vec + up_vecs[i] * wall_height)
			
			
			# RIGHT WALL INNER
			st.add_vertex(points[i] + side_vec)
			st.add_vertex(points[i + 1] + next_side_vec + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i] + side_vec + up_vecs[i] * wall_height)
			st.add_vertex(points[i] + side_vec)
			st.add_vertex(points[i + 1] + next_side_vec)
			st.add_vertex(points[i + 1] + next_side_vec + up_vecs[i + 1] * wall_height)
			
			# TOP
			st.add_vertex(points[i] + side_vec)
			st.add_vertex(points[i] - side_vec)
			st.add_vertex(points[i+1] + next_side_vec)
			st.add_vertex(points[i] - side_vec)
			st.add_vertex(points[i+1] - next_side_vec)
			st.add_vertex(points[i+1] + next_side_vec)
			
			# LEFT WALL OUTER
			st.add_vertex(points[i] - side_vec_wall - up_vecs[i] * wall_thickness)
			st.add_vertex(points[i + 1] - next_side_vec_wall + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i] - side_vec_wall + up_vecs[i] * wall_height)
			st.add_vertex(points[i] - side_vec_wall - up_vecs[i] * wall_thickness)
			st.add_vertex(points[i + 1] - next_side_vec_wall - up_vecs[i + 1] * wall_thickness)
			st.add_vertex(points[i + 1] - next_side_vec_wall + up_vecs[i + 1] * wall_height)
			
			# LEFT WALL TOP
			st.add_vertex(points[i] - side_vec_wall + up_vecs[i] * wall_height)
			st.add_vertex(points[i + 1] - next_side_vec_wall + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i] - side_vec + up_vecs[i] * wall_height)
			st.add_vertex(points[i] - side_vec + up_vecs[i] * wall_height)
			st.add_vertex(points[i + 1] - next_side_vec_wall + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i + 1] - next_side_vec + up_vecs[i + 1] * wall_height)
			
			# LEFT WALL INNER
			st.add_vertex(points[i] - side_vec + up_vecs[i] * wall_height)
			st.add_vertex(points[i + 1] - next_side_vec + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i] - side_vec)
			st.add_vertex(points[i + 1] - next_side_vec + up_vecs[i + 1] * wall_height)
			st.add_vertex(points[i + 1] - next_side_vec)
			st.add_vertex(points[i] - side_vec)
			
			st.generate_normals()
			
	var path := MeshInstance.new()
	path.mesh = st.commit()
	add_child(path)
	
	var body : StaticBody = StaticBody.new()
	var shape : CollisionShape = CollisionShape.new()
	shape.shape = path.mesh.create_trimesh_shape()
	body.add_child(shape)
	add_child(body)
	
